@@ -1,7 +1,7 @@
 from datetime import datetime
 
 from pymobiledevice3.exceptions import DvtDirListError
-from pymobiledevice3.services.dvt.structs import MessageAux
+from pymobiledevice3.services.remote_server import MessageAux
 
 
 class DeviceInfo:
@@ -52,6 +52,9 @@ class DeviceInfo:
     def network_information(self):
         return self.request_information('networkInformation')
 
+    def mach_time_info(self):
+        return self.request_information('machTimeInfo')
+
     def trace_codes(self):
         codes_file = self.request_information('traceCodesFile')
         return {int(k, 16): v for k, v in map(lambda l: l.split(), codes_file.splitlines())}
